<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


/**
 * Class WFFN_API_Update_User_Preference
 *
 * * @extends WP_REST_Controller
 */
if ( ! class_exists( 'WFFN_API_Update_User_Preference' ) ) {
	class WFFN_API_Update_User_Preference extends WP_REST_Controller {

		public static $_instance = null;

		/**
		 * Route base.
		 *
		 * @var string
		 */

		protected $namespace = 'woofunnels-admin';
		protected $rest_base = 'user-preference/(?P<user_id>[\\d]+)';

		public function __construct() {
			add_action( 'rest_api_init', array( $this, 'register_routes' ) );
		}

		public static function get_instance() {
			if ( null === self::$_instance ) {
				self::$_instance = new self;
			}

			return self::$_instance;
		}

		/**
		 * Register the routes for taxes.
		 */
		public function register_routes() {
			register_rest_route( $this->namespace, '/' . $this->rest_base, array(
				'args' => array(
					'user_id' => array(
						'description' => __( 'Unique user id.', 'funnel-builder' ),
						'type'        => 'integer',
					),
				),
				array(
					'methods'             => WP_REST_Server::EDITABLE,
					'callback'            => array( $this, 'update_user_prefernece' ),
					'permission_callback' => array( $this, 'get_api_permission_check' ),
					'args'                => array(
						'user_id' => array(
							'description'       => __( 'Unique user id.', 'funnel-builder' ),
							'type'              => 'integer',
							'required'          => true,
							'validate_callback' => 'rest_validate_request_arg',
						),
					),
				),
			) );
		}

		public function get_api_permission_check() {
			return current_user_can( 'manage_options' );
		}

		/**
		 * @param WP_REST_Request $request
		 *
		 * @return WP_Error|WP_REST_Response
		 */
		public function update_user_prefernece( WP_REST_Request $request ) {
			$user_id         = $request->get_param( 'user_id' );
			$welcome_dismiss = $request->get_param( 'welcome_note_dismiss' );

			$resp = array(
				'status' => false,
				'msg'     => __( 'Failed', 'funnel-builder' )
			);

			if( ! empty( $welcome_dismiss ) && ! empty( $user_id ) ) {
				update_user_meta($user_id, '_wffn_welcome_note_dismissed', $welcome_dismiss );
				$resp = array(
					'status' => true,
					'msg'     => __( 'Success', 'funnel-builder' )
				);
			}


			return rest_ensure_response( $resp );
		}


	}

	if ( ! function_exists( 'wffn_rest_update_user_preference' ) ) {

		function wffn_rest_update_user_preference() {  //@codingStandardsIgnoreLine
			return WFFN_API_Update_User_Preference::get_instance();
		}
	}

	wffn_rest_update_user_preference();
}

